@@ -3,15 +3,19 @@
 
 
 
-msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
+msm.fit = function(params.0, sp, pmethod, suStf, death,
                    Q.diagnostics = FALSE, iterlimsp = iterlimsp, iterlim = iterlim,
-                   sp.method = 'perf',
+                   sp.method = 'perf', approxHess,
                    verbose, tolsp, tolsp.EFS, parallel, no_cores){
 
+  optFun = LikGradHess.general
+  if(approxHess) optFun = LikGradapproxHess.general
+
   stoprule.SP = stoprule.SP.efs = NULL
   conv.sp = Qmatr.diagnostics.list = NULL
   rp = D = L = Sl.sfTemp = St = NULL # will be not null only if efs is used
   bs.mgfit = magpp = wor.c = NULL
+  iter.sp = iter.inner = NULL
 
   # Unpack necessary variables -------------------------------------------------------------------
   data = suStf$data # HERE WE USED TO HAVE $data BUT I THINK THAT IN ALL OTHER TESTS I NORMALLY DO data AND data.long COINCIDE (not in no intercept test though)
@@ -31,19 +35,19 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
   silent.trust = FALSE
 
 
-  # Fixing S.list
-  S.list = suStf$S.list
-  # Keep an eye on whether this fix is OK - this is just to remove NULL, may not be necessary any more
-  S.list.temp = list()
-  ii.fix = 1
-  for(i.fix in 1:length(S.list)){
-    if(!is.null(S.list[[i.fix]])){
-      S.list.temp[[ii.fix]] = S.list[[i.fix]]
-      ii.fix = ii.fix + 1
-    }
-  }
-  S.list = S.list.temp
-  rm(S.list.temp)
+  # # Fixing S.list [MOVED WITHIN THE SMOOTHING LOOP - keep an eye on correctness of this]
+  # S.list = suStf$S.list
+  # # Keep an eye on whether this fix is OK - this is just to remove NULL, may not be necessary any more
+  # S.list.temp = list()
+  # ii.fix = 1
+  # for(i.fix in 1:length(S.list)){
+  #   if(!is.null(S.list[[i.fix]])){
+  #     S.list.temp[[ii.fix]] = S.list[[i.fix]]
+  #     ii.fix = ii.fix + 1
+  #   }
+  # }
+  # S.list = S.list.temp
+  # rm(S.list.temp)
 
   start.pos.par.only.smooth = suStf$start.pos.par.only.smooth
   start.pos.par.only.smooth.FPC = suStf$start.pos.par.only.smooth.FPC
@@ -68,6 +72,7 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
   # ----------------------------------------------------------------------------------------------
 
 
+
   # ***** OPTIMIZATION STARTS HERE *****
 
   if(Q.diagnostics) Qmatr.diagnostics.list = list()
@@ -75,15 +80,15 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
   ii = 1
   fit.all = list()
 
-  fit <- try(trust::trust(LikGradHess.general, params.0, rinit = 1, # the following are all params needed by the function ********
+  fit <- try(trust::trust(optFun, params.0, rinit = 1, # the following are all params needed by the function ********
              data = data, full.X = full.X, MM = MM, pen.matr.S.lambda = pen.matr.S.lambda, # *****
               aggregated.provided = FALSE, do.gradient = TRUE, do.hessian = TRUE,
-             pmethod = pmethod, death = death,
+             pmethod = pmethod, death = death, verbose = verbose,
              Qmatr.diagnostics.list = Qmatr.diagnostics.list,
              parallel = parallel, no_cores = no_cores,
              rmax = 100, parscale = rep(1, length(params.0)), blather = TRUE, iterlim = iterlim), silent = silent.trust)
 
-  if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n.', ii, 'to trust region algorithm. Look into partial results for diagnostics.')
+  if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n. ', ii, ' to trust region algorithm. Look into partial results for diagnostics.')
 
 
 
@@ -93,7 +98,21 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
   sp.all = c(sp)
 
 
-  if(length(sp) > 0){ # i.e. only do smoothing parameters estimation if there are smooths! (change/check condition)
+  if( length(sp) > 0 ){ # if length is 0, this suppresses smoothing parameter estimation
+
+    # Fixing S.list
+    S.list = suStf$S.list
+    # Keep an eye on whether this fix is OK - this is just to remove NULL, may not be necessary any more
+    S.list.temp = list()
+    ii.fix = 1
+    for(i.fix in 1:length(S.list)){
+      if(!is.null(S.list[[i.fix]])){
+        S.list.temp[[ii.fix]] = S.list[[i.fix]]
+        ii.fix = ii.fix + 1
+      }
+    }
+    S.list = S.list.temp
+    rm(S.list.temp)
 
     # ********************************************* #
     # MAGIC BASED SMOOTHING PARAMETER ESTIMATION ####
@@ -133,15 +152,15 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
         # ***** With Q computation put internally *****
         if(Q.diagnostics) Qmatr.diagnostics.list = fit$Qmatr.diagnostics.list
 
-        fit <- try(trust::trust(LikGradHess.general, o.ests, rinit = 1, # the following are all params needed by the function ********
+        fit <- try(trust::trust(optFun, o.ests, rinit = 1, # the following are all params needed by the function ********
                      data = data, full.X = full.X, MM = MM, pen.matr.S.lambda = pen.matr.S.lambda, # *****
                       aggregated.provided = FALSE, do.gradient = TRUE, do.hessian = TRUE,
-                     pmethod = pmethod, death = death,
+                     pmethod = pmethod, death = death, verbose = verbose,
                      Qmatr.diagnostics.list = Qmatr.diagnostics.list,
                      parallel = parallel, no_cores = no_cores, # **************************************
                      rmax = 100, parscale = rep(1, length(params.0)), blather = TRUE, iterlim = iterlim), silent = silent.trust)
 
-        if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n.', ii, 'to trust region algorithm. Look into partial results for diagnostics.')
+        if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n. ', ii, ' to trust region algorithm. Look into partial results for diagnostics.')
 
         fit.all[[ii]] = fit
         ii = ii + 1
@@ -335,16 +354,16 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
 
 
         if(Q.diagnostics) Qmatr.diagnostics.list = fit$Qmatr.diagnostics.list
-        fit <- try(trust::trust(LikGradHess.general, o.ests, rinit = 1, # the following are all params needed by the function ********
+        fit <- try(trust::trust(optFun, o.ests, rinit = 1, # the following are all params needed by the function ********
                      data = data, full.X = full.X, MM = MM, pen.matr.S.lambda = pen.matr.S.lambda, # *****
                       aggregated.provided = FALSE, do.gradient = TRUE, do.hessian = TRUE,
-                     pmethod = pmethod, death = death,
+                     pmethod = pmethod, death = death, verbose = verbose,
                      Qmatr.diagnostics.list = Qmatr.diagnostics.list,
                      parallel = parallel, no_cores = no_cores, # *****
                      rmax = 100, parscale = rep(1, length(params.0)), blather = TRUE, iterlim = iterlim), silent = silent.trust)
 
 
-        if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n.', ii, 'to trust region algorithm. Look into partial results for diagnostics.')
+        if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n. ', ii, ' to trust region algorithm. Look into partial results for diagnostics.')
 
         iter.inner <- iter.inner + fit$iterations
         rp <- ldetS(Sl.sf, rho = lsp1, fixed = rep(FALSE, length(lsp1)), np = length(fit$argument), root = TRUE)
@@ -367,16 +386,16 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
             pen.matr.S.lambda = pen.matr.S.lambda[1:max(pos.optparams2) == pos.optparams2, 1:max(pos.optparams2) == pos.optparams2]
 
             if(Q.diagnostics) Qmatr.diagnostics.list = fit$Qmatr.diagnostics.list
-            fit2 <- try(trust::trust(LikGradHess.general, o.ests, rinit = 1, # the following are all params needed by the function ********
+            fit2 <- try(trust::trust(optFun, o.ests, rinit = 1, # the following are all params needed by the function ********
                           data = data, full.X = full.X, MM = MM, pen.matr.S.lambda = pen.matr.S.lambda, # *****
                            aggregated.provided = FALSE, do.gradient = TRUE, do.hessian = TRUE,
-                          pmethod = pmethod, death = death,
+                          pmethod = pmethod, death = death, verbose = verbose,
                           Qmatr.diagnostics.list = Qmatr.diagnostics.list,
                           parallel = parallel, no_cores = no_cores, # *****
                           rmax = 100, parscale = rep(1, length(params.0)), blather = TRUE, iterlim = iterlim), silent = silent.trust)
 
 
-            if(!is.null(attr(fit2$error, 'class'))) warning('Unable to carry out call n.', ii, 'to trust region algorithm. Look into partial results for diagnostics.')
+            if(!is.null(attr(fit2$error, 'class'))) warning('Unable to carry out call n. ', ii, ' to trust region algorithm. Look into partial results for diagnostics.')
 
 
             iter.inner <- iter.inner + fit2$iterations
@@ -421,16 +440,16 @@ msm.fit = function(params.0, sp, pen.matr.S.lambda, pmethod, suStf, death,
 
             if(Q.diagnostics) Qmatr.diagnostics.list = fit$Qmatr.diagnostics.list
 
-            fit <- try(trust::trust(LikGradHess.general, o.ests, rinit = 1, # the following are all params needed by the function ********
+            fit <- try(trust::trust(optFun, o.ests, rinit = 1, # the following are all params needed by the function ********
                          data = data, full.X = full.X, MM = MM, pen.matr.S.lambda = pen.matr.S.lambda, # *****
                           aggregated.provided = FALSE, do.gradient = TRUE, do.hessian = TRUE,
-                         pmethod = pmethod, death = death,
+                         pmethod = pmethod, death = death, verbose = verbose,
                          Qmatr.diagnostics.list = Qmatr.diagnostics.list,
                          parallel = parallel, no_cores = no_cores, # *****
                          rmax = 100, parscale = rep(1, length(params.0)), blather = TRUE, iterlim = iterlim), silent = silent.trust)
 
 
-            if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n.', ii, 'to trust region algorithm. Look into partial results for diagnostics.')
+            if(!is.null(attr(fit$error, 'class'))) warning('Unable to carry out call n. ', ii, ' to trust region algorithm. Look into partial results for diagnostics.')
 
             iter.inner <- iter.inner + fit$iterations
             rp <- ldetS(Sl.sf, rho = lsp1, fixed = rep(FALSE, length(lsp1)), np = length(fit$argument), root = TRUE)
